from dataclasses import dataclass
from typing import Optional, List
from flask_login import UserMixin

@dataclass
class CurrentUser(UserMixin):
    """当前用户信息结构体"""
    user_id: Optional[int] = None
    username: Optional[str] = None
    name: Optional[str] = None
    roles: List[str] = None

    def __post_init__(self):
        """初始化角色列表"""
        if self.roles is None:
            self.roles = []

    def get_id(self):
        """实现Flask-Login要求的get_id方法"""
        return str(self.user_id) if self.user_id else None

    @property
    def is_authenticated(self) -> bool:
        """检查用户是否已认证

        Returns:
            bool: 如果用户已认证返回True，否则返回False
        """
        return self.user_id is not None and self.username is not None

    @property
    def is_super_admin(self) -> bool:
        """检查用户是否为超级管理员

        Returns:
            bool: 如果用户是超级管理员返回True，否则返回False
        """
        return 'super_admin' in self.roles

    @property
    def is_admin(self) -> bool:
        """检查用户是否为超级管理员

        Returns:
            bool: 如果用户是超级管理员返回True，否则返回False
        """
        return self.username == 'admin' or 'admin' in self.roles

    @property
    def is_sys(self) -> bool:
        """检查用户是否为系统管理员

        Returns:
            bool: 如果用户是系统管理员返回True，否则返回False
        """
        return self.is_admin or 'sys' in self.roles

    @property
    def is_edit(self) -> bool:
        """检查用户是否有编辑权限

        Returns:
            bool: 如果用户有编辑权限返回True，否则返回False
        """
        return self.is_admin or 'edit' in self.roles


    def has_role(self, role: str) -> bool:
        """检查用户是否拥有指定角色

        Args:
            role (str): 角色名称

        Returns:
            bool: 如果用户拥有指定角色返回True，否则返回False
        """
        return role in self.roles
    
    def has_any_role(self, roles: List[str]) -> bool:
        """检查用户是否拥有指定角色列表中的任意一个角色

        Args:
            roles (List[str]): 角色名称列表

        Returns:
            bool: 如果用户拥有指定角色列表中的任意一个角色返回True，否则返回False
        """
        return any(role in self.roles for role in roles)
    
    def has_all_roles(self, roles: List[str]) -> bool:
        """检查用户是否拥有指定角色列表中的所有角色

        Args:
            roles (List[str]): 角色名称列表

        Returns:
            bool: 如果用户拥有指定角色列表中的所有角色返回True，否则返回False
        """
        return all(role in self.roles for role in roles)
