from flask import session
from typing import Optional
from .current_user import CurrentUser


class UserSession:
    """用户会话管理类"""

    @staticmethod
    def set_user(user_id: int, username: str, name: str, roles: Optional[list] = None) -> None:
        """设置用户登录状态

        Args:
            user_id (int): 用户ID
            username (str): 用户名
            name (str): 用户姓名
            roles (Optional[list], optional): 用户角色列表. Defaults to None.
        """
        session['user_id'] = user_id
        session['username'] = username
        session['name'] = name
        session['roles'] = roles or []

    @staticmethod
    def get_current_user() -> CurrentUser:
        """获取当前登录用户信息

        Returns:
            CurrentUser: 返回当前用户信息结构体
        """
        return CurrentUser(
            user_id=session.get('user_id'),
            username=session.get('username'),
            name=session.get('name'),
            roles=session.get('roles', [])
        )
    
    @staticmethod
    def get_current_username() -> Optional[str]:
        """获取当前登录用户名

        Returns:
            Optional[str]: 返回用户名，未登录则返回None
        """
        return session.get('username')
    
    @staticmethod
    def get_current_user_id() -> Optional[int]:
        """获取当前登录用户ID

        Returns:
            Optional[int]: 返回用户ID，未登录则返回None
        """
        return session.get('user_id')
    
    @staticmethod
    def get_current_name() -> Optional[str]:
        """获取当前登录用户姓名

        Returns:
            Optional[str]: 返回用户姓名，未登录则返回None
        """
        return session.get('name')

    
    @staticmethod
    def clear_user() -> None:
        """清除用户登录状态"""
        session.pop('user_id', None)
        session.pop('username', None)
        session.pop('name', None)
        session.pop('roles', None)
    
    @staticmethod
    def is_logged_in() -> bool:
        """检查用户是否已登录

        Returns:
            bool: 如果用户已登录返回True，否则返回False
        """
        return 'user_id' in session and 'username' in session
    
    @staticmethod
    def is_admin() -> bool:
        """检查当前用户是否为管理员

        Returns:
            bool: 如果当前用户是管理员返回True，否则返回False
        """
        return session.get('username') == 'admin'
    
    @staticmethod
    def get_current_roles() -> list:
        """获取当前用户角色列表

        Returns:
            list: 返回用户角色列表，未登录则返回空列表
        """
        return session.get('roles', [])

    @staticmethod
    def has_role(role: str) -> bool:
        """检查当前用户是否拥有指定角色

        Args:
            role (str): 角色名称

        Returns:
            bool: 如果用户拥有指定角色返回True，否则返回False
        """
        return role in session.get('roles', [])

    @staticmethod
    def has_all_roles(roles: list) -> bool:
        """检查当前用户是否拥有指定角色列表中的所有角色

        Args:
            roles (list): 角色名称列表

        Returns:
            bool: 如果用户拥有指定角色列表中的所有角色返回True，否则返回False
        """
        user_roles = session.get('roles', [])
        return all(role in user_roles for role in roles)