from typing import Dict, Optional, Any, List, Tuple, Generator
from contextlib import contextmanager
import threading
import tools.utils as utils
from sqlalchemy.orm import sessionmaker, declarative_base

# 创建基础模型类
Base = declarative_base()

class DBHelper:
    _instance = None
    _lock = threading.Lock()
    _main_config_key = ""
    def __new__(cls, *args, **kwargs):
        with cls._lock:
            if cls._instance is None:
                cls._instance = super(DBHelper, cls).__new__(cls)
                cls._instance._initialized = False
            return cls._instance
    
    def __init__(self):
        if self._initialized:
            return
            
        self._config_cache: Dict[str, Dict[str, str]] = {}
        self._engines: Dict[str, Any] = {}
        self._sessions: Dict[str, sessionmaker] = {}
        self._default_config = {
            'pool_size': 5,
            'max_overflow': 10,
            'pool_timeout': 30,
            'pool_recycle': 3600
        }
        self._initialized = True
        
    def set_default_config(self, config: Dict[str, str]) -> None:
        """设置默认连接配置"""
        self._default_config.update(config)
    
    def get_config_for_database(self, database: str) -> Dict[str, str]:
        """获取数据库的连接配置
        
        按照以下顺序查找配置：
        1. 配置缓存
        2. 配置文件中特定数据库的配置
        3. 配置文件中的main_config配置
        4. 默认配置
        
        Args:
            database: 数据库名称
            
        Returns:
            数据库连接配置
            
        Raises:
            Exception: 当找不到特定数据库配置且main_config配置也不存在时
        """
        if database in self._config_cache:
            return self._config_cache[database]
        
        db_config = utils.get_config_object(f"db.{database}")
        if db_config:
            self._config_cache[database] = db_config
            return db_config
        
        main_config = utils.get_config_object(self._main_config_key)
        if not main_config:
            raise Exception(f"未找到数据库 {database} 的配置，且main_config配置不存在")
        
        self._config_cache[database] = main_config
        return main_config
    
    def execute_query(self, database: str, query: str, params: Optional[Any] = None) -> List[Tuple]:
        """执行查询并返回结果
        
        Args:
            database: 数据库名称
            query: SQL查询语句
            params: 查询参数
            
        Returns:
            查询结果列表
        """
        raise NotImplementedError("子类必须实现execute_query方法")
    
    def execute_non_query(self, database: str, query: str, params: Optional[Any] = None) -> int:
        """执行非查询操作（如INSERT, UPDATE, DELETE）
        
        Args:
            database: 数据库名称
            query: SQL语句
            params: 查询参数
            
        Returns:
            受影响的行数
        """
        raise NotImplementedError("子类必须实现execute_non_query方法")
    
    def execute_scalar(self, database: str, query: str, params: Optional[Any] = None) -> Any:
        """执行查询并返回第一行第一列的值
        
        Args:
            database: 数据库名称
            query: SQL查询语句
            params: 查询参数
            
        Returns:
            查询结果的第一行第一列的值
        """
        raise NotImplementedError("子类必须实现execute_scalar方法")
    
    def execute_procedure(self, database: str, procedure_name: str, params: Optional[Dict[str, Any]] = None) -> List[Tuple]:
        """执行存储过程
        
        Args:
            database: 数据库名称
            procedure_name: 存储过程名称
            params: 存储过程参数
            
        Returns:
            存储过程执行结果
        """
        raise NotImplementedError("子类必须实现execute_procedure方法")
    
    def get_engine(self, database: str, config: Optional[Dict[str, Any]] = None) -> Any:
        """获取或创建数据库引擎
        
        Args:
            database: 数据库名称
            config: 数据库配置信息
            
        Returns:
            SQLAlchemy引擎实例
        """
        raise NotImplementedError("子类必须实现get_engine方法")
    
    def get_session_maker(self, database: str, config: Optional[Dict[str, Any]] = None) -> sessionmaker:
        """获取或创建会话工厂
        
        Args:
            database: 数据库名称
            config: 数据库配置信息
            
        Returns:
            会话工厂实例
        """
        if database in self._sessions:
            return self._sessions[database]
        
        engine = self.get_engine(database, config)
        session = sessionmaker(bind=engine)
        self._sessions[database] = session
        return session
    
    @contextmanager
    def session_scope(self, database: str, config: Optional[Dict[str, Any]] = None) -> Generator:
        """创建会话的上下文管理器
        
        Args:
            database: 数据库名称
            config: 数据库配置信息
            
        Yields:
            数据库会话
        """
        session = self.get_session_maker(database, config)
        session = session()
        try:
            yield session
            session.commit()
        except:
            session.rollback()
            raise
        finally:
            session.close()
    
    def dispose_engine(self, database: str) -> None:
        """释放指定数据库的引擎资源
        
        Args:
            database: 数据库名称
        """
        if database in self._engines:
            self._engines[database].dispose()
            del self._engines[database]
        if database in self._sessions:
            del self._sessions[database]
    
    def dispose_all(self) -> None:
        """释放所有数据库资源"""
        for database in list(self._engines.keys()):
            self.dispose_engine(database)
    
    def __del__(self):
        """析构函数，确保所有资源被释放"""
        self.dispose_all()