import pymysql
from typing import Dict, Optional, Any, List, Tuple
from contextlib import contextmanager
from sqlalchemy import create_engine
from sqlalchemy.engine import Engine
from .base import DBHelper
from pymysql import Error

class MySQLHelper(DBHelper):
    def __init__(self):
        super().__init__()
        self._connections: Dict[str, pymysql.Connection] = {}
        self._default_config = {
            'db': '',
            'host': 'localhost',
            'port': 3306,
            'user': '',
            'password': '',
            'charset': 'utf8mb4'
        }
        self._main_config_key = "db.mysql_main"

    def get_engine(self, database: str, config: Optional[Dict[str, Any]] = None) -> Engine:
        """获取或创建数据库引擎
        
        Args:
            database: 数据库名称
            config: 可选的连接配置
            
        Returns:
            SQLAlchemy引擎实例
        """
        if database in self._engines:
            return self._engines[database]
        
        conn_config = self._default_config.copy()
        db_config = self.get_config_for_database(database)
        conn_config.update(db_config)
        if config:
            conn_config.update(config)
        
        if 'db' not in conn_config or not conn_config['db']:
            conn_config['db'] = database
        
        url = f"mysql+pymysql://{conn_config['user']}:{conn_config['password']}@{conn_config['host']}:{conn_config['port']}/{conn_config['db']}"
        self._engines[database] = create_engine(
            url,
            pool_size=self._default_config['pool_size'],
            max_overflow=self._default_config['max_overflow'],
            pool_timeout=self._default_config['pool_timeout'],
            pool_recycle=self._default_config['pool_recycle']
        )
        return self._engines[database]
    
    def connect(self, database: str, config: Optional[Dict[str, str]] = None) -> pymysql.Connection:
        """连接到指定数据库
        
        Args:
            database: 数据库名称
            config: 可选的连接配置，如果不提供则使用默认配置
            
        Returns:
            数据库连接对象
        """
        if database in self._connections:
            try:
                self._connections[database].ping(reconnect=True)
                return self._connections[database]
            except Error:
                try:
                    self._connections[database].close()
                except Error:
                    pass
                del self._connections[database]
        
        conn_config = self._default_config.copy()
        db_config = self.get_config_for_database(database)
        conn_config.update(db_config)
        if config:
            conn_config.update(config)
        
        if 'db' not in conn_config or not conn_config['db']:
            conn_config['db'] = database
        
        connection = pymysql.connect(**conn_config)
        self._connections[database] = connection
        return connection
    
    @contextmanager
    def connection(self, database: str):
        """获取数据库连接的上下文管理器
        
        Args:
            database: 数据库名称
            
        Yields:
            数据库连接对象
        """
        connection = self.connect(database)
        try:
            yield connection
        finally:
            pass
    
    def execute_query(self, database: str, query: str, params: Optional[Tuple] = None) -> List[Tuple]:
        connection = self.connect(database)
        cursor = connection.cursor()
        
        try:
            if params:
                cursor.execute(query, params)
            else:
                cursor.execute(query)
            
            results = cursor.fetchall()
            return [tuple(row) for row in results]
        finally:
            cursor.close()
    
    def execute_non_query(self, database: str, query: str, params: Optional[Tuple] = None) -> int:
        connection = self.connect(database)
        cursor = connection.cursor()
        
        try:
            if params:
                cursor.execute(query, params)
            else:
                cursor.execute(query)
            
            connection.commit()
            return cursor.rowcount
        finally:
            cursor.close()
    
    def execute_scalar(self, database: str, query: str, params: Optional[Tuple] = None) -> Any:
        connection = self.connect(database)
        cursor = connection.cursor()
        
        try:
            if params:
                cursor.execute(query, params)
            else:
                cursor.execute(query)
            
            row = cursor.fetchone()
            return row[0] if row else None
        finally:
            cursor.close()
    
    def execute_procedure(self, database: str, procedure_name: str, params: Optional[Dict[str, Any]] = None) -> List[Tuple]:
        connection = self.connect(database)
        cursor = connection.cursor()
        
        try:
            if params:
                param_placeholders = ", ".join([f"%s" for _ in params.keys()])
                query = f"CALL {procedure_name}({param_placeholders})"
                cursor.execute(query, list(params.values()))
            else:
                cursor.execute(f"CALL {procedure_name}()")
            
            results = cursor.fetchall()
            return [tuple(row) for row in results]
        finally:
            cursor.close()
    
    def dispose_all(self) -> None:
        """释放所有数据库连接"""
        for conn in self._connections.values():
            try:
                conn.close()
            except Error:
                pass
        self._connections.clear()
    
    def __del__(self):
        """析构函数，确保所有连接被关闭"""
        self.dispose_all()